@@ -103,12 +103,13 @@ public:
     struct element_block_node_type
     {
         mtm::element_t type;
+        size_t offset;
         size_type size;
         const element_block_type* data;
 
-        element_block_node_type() : type(mtm::element_empty), size(0), data(nullptr) {}
+        element_block_node_type() : type(mtm::element_empty), offset(0), size(0), data(nullptr) {}
         element_block_node_type(const element_block_node_type& other) :
-            type(other.type), size(other.size), data(other.data) {}
+            type(other.type), offset(other.offset), size(other.size), data(other.data) {}
     };
 
     static mtm::element_t to_mtm_type(mdds::mtv::element_t mtv_type)
@@ -531,6 +532,23 @@ public:
     template<typename _Func>
     void walk(_Func& func) const;
 
+    /**
+     * Walk through the element blocks that make up the sub-matrix defined by
+     * start and end positions.
+     *
+     * @param func function object whose operator() gets called on the
+     *              element block.
+     *
+     * @param start the column/row position of the upper-left corner of the
+     *              sub-matrix.
+     *
+     * @param end the column/row position of the lower-right corner of the
+     *          sub-matrix.  Both column and row must be greater or equal to
+     *          those of the start position.
+     */
+    template<typename _Func>
+    void walk(_Func& func, const size_pair_type& start, const size_pair_type& end) const;
+
 #ifdef MDDS_MULTI_TYPE_MATRIX_DEBUG
     void dump() const
     {
