@@ -47,14 +47,49 @@ using std::endl;
 
 namespace mdds {
 
+namespace detail {
+
 /**
- * Multi-type vector consists of a series of blocks, and each block stores a
- * series of non-empty elements of identical type.  In this container,
- * elements are represented simply as values that they store; there are no
- * separate element objects that the user of this container needs to deal
- * with.  The user accesses directly with the raw values.
+ * Empty event function handler structure, used when no custom function
+ * handler is specified.
+ *
+ * @see mdds::multi_type_vector
  */
-template<typename _ElemBlockFunc>
+struct mtv_event_func
+{
+    void element_block_acquired(const mdds::mtv::base_element_block* /*block*/) {}
+
+    void element_block_released(const mdds::mtv::base_element_block* /*block*/) {}
+};
+
+}
+
+/**
+ * Multi-type vector consists of a series of one or more blocks, and each
+ * block may either be empty, or stores a series of non-empty elements of
+ * identical type.  These blocks collectively represent a single
+ * one-dimension array that may store elements of different types.  It is
+ * guaranteed that the block types of neighboring blocks are always
+ * different.
+ *
+ * Structurally, the primary array stores block instances whose types
+ * are of <code>value_type</code>, which in turn consists of the following
+ * data members:
+ *
+ * <ul>
+ * <li><code>type</code> which indicates the block type,</li>
+ * <li><code>position</code> which stores the logical position of the
+ * first element of the block,</li>
+ * <li><code>size</code> which stores the logical size of the block,
+ * and</li>
+ * <li><code>data</code> which stores the pointer to a secondary array
+ * (a.k.a. element block) which stores the actual element values, or
+ * <code>nullptr</code> in case the block represents an empty segment.</li>
+ * </ul>
+ *
+ * @see mdds::multi_type_vector::value_type
+ */
+template<typename _ElemBlockFunc, typename _EventFunc = detail::mtv_event_func>
 class multi_type_vector
 {
 public:
@@ -64,6 +99,25 @@ public:
     typedef typename mdds::mtv::element_t element_category_type;
     typedef _ElemBlockFunc element_block_func;
 
+    /**
+     * Optional event handler function structure, whose functions get called
+     * at specific events.  The following events are currently supported:
+     *
+     * <ul>
+     * <li><code>element_block_acquired</code> - this gets called whenever
+     * the container acquires a new element block either as a result of a new
+     * element block creation or a tranfer of an existing
+     * element block from another container.</li>
+     * <li><code>element_block_released</code> - this gets called whenever
+     * the container releases an existing element block either because
+     * the block gets deleted or gets transferred to another container.</li>
+     * </ul>
+     *
+     * @see mdds::detail::mtv_event_func for the precise function signatures
+     *      of the event handler functions.
+     */
+    typedef _EventFunc event_func;
+
 private:
 
     struct block
@@ -135,6 +189,21 @@ public:
     typedef __mtv::const_iterator_base<const_iterator_trait, itr_forward_update, iterator> const_iterator;
     typedef __mtv::const_iterator_base<const_reverse_iterator_trait, itr_no_update, reverse_iterator> const_reverse_iterator;
 
+    /**
+     * value_type is the type of a block stored in the primary array.  It
+     * consists of the following data members:
+     *
+     * <ul>
+     * <li><code>type</code> which indicates the block type,</li>
+     * <li><code>position</code> which stores the logical position of the
+     * first element of the block,</li>
+     * <li><code>size</code> which stores the logical size of the block,
+     * and</li>
+     * <li><code>data</code> which stores the pointer to a secondary array
+     * (a.k.a. element block) which stores the actual element values, or
+     * <code>nullptr</code> in case the block represents an empty segment.</li>
+     * </ul>
+     */
     typedef itr_node value_type;
 
     typedef std::pair<iterator, size_type> position_type;
@@ -204,11 +273,30 @@ public:
     const_reverse_iterator rbegin() const;
     const_reverse_iterator rend() const;
 
+    event_func& event_handler();
+    const event_func& event_handler() const;
+
     /**
      * Default constructor.  It initializes the container with empty size.
      */
     multi_type_vector();
 
+    /**
+     * Constructor that takes an lvalue reference to an event handler object.
+     * The event handler instance will be copy-constructed.
+     *
+     * @param hdl lvalue reference to an event handler object.
+     */
+    multi_type_vector(const event_func& hdl);
+
+    /**
+     * Constructor that takes an rvalue reference to an event handler object.
+     * The event handler instance will be move-constructed.
+     *
+     * @param hdl rvalue reference to an event handler object.
+     */
+    multi_type_vector(event_func&& hdl);
+
     /**
      * Constructor that takes initial size of the container.  When the size
      * specified is greater than 0, it initializes the container with empty
@@ -932,6 +1020,29 @@ public:
 
 private:
 
+    /**
+     * Delete only the element block owned by an outer block.
+     *
+     * @param p pointer to the instance of an outer block that may own an
+     *          element block instance.
+     */
+    void delete_element_block(block* p);
+
+    /**
+     * Delete an outer block.
+     *
+     * @param p pointer to the instance of an outer block to delete.
+     */
+    void delete_block(block* p);
+
+    /**
+     * Delete one or more outer blocks in the specified iterator ranges.
+     *
+     * @param it start position.
+     * @param it_end end position (not inclusive).
+     */
+    void delete_blocks(typename blocks_type::iterator it, typename blocks_type::iterator it_end);
+
     template<typename _T>
     iterator set_impl(size_type pos, size_type start_row, size_type block_index, const _T& value);
 
@@ -966,7 +1077,7 @@ private:
     void get_block_position(const const_iterator& pos_hint, size_type pos, size_type& start_pos, size_type& block_index) const;
 
     template<typename _T>
-    static void create_new_block_with_new_cell(element_block_type*& data, const _T& cell);
+    void create_new_block_with_new_cell(element_block_type*& data, const _T& cell);
 
     template<typename _T>
     iterator set_cell_to_middle_of_block(
@@ -1020,7 +1131,7 @@ private:
         size_type start_pos_in_block1, size_type block_index1, size_type start_pos_in_block2, size_type block_index2,
         size_type start_pos_in_dblock1, size_type dblock_index1, size_type start_pos_in_dblock2, size_type dblock_index2);
 
-    void swap_single_blocks(
+    void swap_single_block(
         multi_type_vector& other, size_type start_pos, size_type end_pos, size_type other_pos,
         size_type start_pos_in_block, size_type block_index, size_type start_pos_in_other_block, size_type other_block_index);
 
@@ -1034,6 +1145,8 @@ private:
         size_type start_pos_in_block1, size_type block_index1, size_type start_pos_in_block2, size_type block_index2,
         size_type start_pos_in_dblock1, size_type dblock_index1, size_type start_pos_in_dblock2, size_type dblock_index2);
 
+    void insert_blocks_at(size_type insert_pos, blocks_type& new_blocks);
+
     void prepare_blocks_to_transfer(blocks_to_transfer& bucket, size_type block_index1, size_type offset1, size_type block_index2, size_type offset2);
 
     iterator set_whole_block_empty(size_type block_index, size_type start_pos_in_block, bool overwrite);
@@ -1136,6 +1249,14 @@ private:
         size_type block_index, size_type offset, size_type new_block_size, bool overwrite);
 
     block* get_previous_block_of_type(size_type block_index, element_category_type cat);
+
+    /**
+     * @param block_index index of the current block.
+     * @param cat desired block type.
+     *
+     * @return pointer to the next block if the next block exists and it's of
+     *         specified type, otherwise nullptr will be returned.
+     */
     block* get_next_block_of_type(size_type block_index, element_category_type cat);
 
     /**
@@ -1180,6 +1301,7 @@ private:
     }
 
 private:
+    event_func m_hdl_event;
     blocks_type m_blocks;
     size_type m_cur_size;
 };
